#ifdef __cplusplus
extern "C" {
#else
#include <stdbool.h>
#endif

/**
 * A wrapper of the C++ CaosDBConnection class.
 *
 * We use a wrapper for future extensibility and in order to have a minimal
 * capability for type checking in C even though the C++ class
 * CaosDBConnection is opaque in C.
 */
typedef struct {
  void *wrapped_connection;
} caosdb_connection_connection;

/**
 * A wrapper of the C++ CaosDBConnectionConfig class.
 *
 * We use a wrapper for future extensibility and in order to have a minimal
 * capability for type checking in C even though the C++ class
 * CaosDBConnection is opaque in C.
 */
typedef struct {
  void *wrapped_connection_config;
} caosdb_connection_connection_config;

/**
 * A wrapper of the C++ VersionInfo class.
 *
 * We use a wrapper for future extensibility and in order to have a minimal
 * capability for type checking in C even though the C++ class
 * CaosDBConnection is opaque in C.
 */
typedef struct {
  void *wrapped_version_info;
} caosdb_info_version_info;

/**
 * Return the environment variable of the given name.
 *
 * If the environment variable is not set, return the fall_back instead.
 */
const char *caosdb_utils_get_env_var(const char *name, const char *fall_back);

/**
 * Create a connection config.
 *
 * The config is needed to instantiate a connection.
 *
 * The config is ready to be used but you might want use
 * caosdb_connection_config_add_cacert and
 * caosdb_connection_config_add_plain_authenticator to specify more options of
 * the connection.
 */
int caosdb_connection_create_config(caosdb_connection_connection_config *config,
                                    const char *host, const int port,
                                    const bool tls);

/**
 * Add a public certificate of a trusted certificate authority to an
 * existing, tls-enabled connection configuration.
 *
 * @param cacert path to a pem-file.
 */
int caosdb_connection_config_add_cacert(
  caosdb_connection_connection_config *config, const char *cacert);

/**
 * Add a plain text authenticator to an existing, tls enabled connection
 * configuration.
 */
int caosdb_connection_config_add_plain_authenticator(
  caosdb_connection_connection_config *config, const char *username,
  const char *password);

/**
 * Create a connection instance.
 *
 * The connection is needed to create transactions and to initiate any other
 * interaction with a CaosDB server.
 */
int caosdb_connection_create_connection(
  caosdb_connection_connection *connection,
  const caosdb_connection_connection_config *config);

/**
 * Request the version of the server.
 */
int caosdb_connection_get_version_info(
  caosdb_info_version_info *version_info,
  const caosdb_connection_connection *connection);

#ifdef __cplusplus
}
#endif
